@@ -1,4 +1,4 @@
-#  $Id: V01.pm,v 1.17 2000/10/22 10:59:00 aigan Exp $  -*-cperl-*-
+#  $Id: V01.pm,v 1.22 2000/11/10 18:41:37 aigan Exp $  -*-cperl-*-
 
 package RDF::Service::Interface::DBI::V01;
 
@@ -26,7 +26,7 @@ use Time::Object;
 use Date::Parse;
 use vars qw( $prefix $interface_uri @node_fields );
 use RDF::Service::Constants qw( :all );
-use RDF::Service::Cache qw( get_unique_id uri2id id2uri debug );
+use RDF::Service::Cache qw( get_unique_id uri2id id2uri debug $DEBUG );
 use RDF::Service::Resource;
 use Data::Dumper;
 use Carp;
@@ -56,7 +56,7 @@ sub register
 
     my $dbi_options =
     {
-	RaiseError => 1,
+	RaiseError => 0,
     };
 
     my $dbh = ( DBI->connect( $connect, $name, $passwd, $dbi_options ) );
@@ -76,7 +76,7 @@ sub register
     # interface objects but the same interface module.
     #
     debug "Store DBH for $i->[URISTR] in ".
-	"[PRIVATE]{$i->[ID]}{'dbh'}\n", 2;
+	"[PRIVATE]{$i->[ID]}{'dbh'}\n", 3;
 
     $i->[PRIVATE]{$i->[ID]}{'dbh'} = $dbh;
 
@@ -84,14 +84,14 @@ sub register
     {
 	'' =>
 	{
-	    NS_L.'#Service' =>
+	    NS_LS.'#Service' =>
 	    {
 	    },
-	    NS_L.'#interface' =>
+	    NS_LS.'#interface' =>
 	    {
 		#'list_arcs' => [\&list_arcs],
 	    },
-	    NS_L.'#Model' =>
+	    NS_LS.'#Model' =>
 	    {
 		'create_model'    => [\&create_model],
 		'add_arc'        => [\&add_arc],
@@ -100,8 +100,8 @@ sub register
 	    NS_RDFS.'Resource'   =>
 	    {
 		'init_types'     => [\&init_types],
-		'init_props'     => [\&init_props],
-		'init_rev_props' => [\&init_rev_props],
+		'init_rev_subjs' => [\&init_rev_subjs],
+		'init_rev_objs'  => [\&init_rev_objs],
 		'name'           => [\&name],
 		'find_node'      => [\&find_node],
 		'create_literal' => [\&create_literal],
@@ -146,7 +146,7 @@ sub find_node
     $sth->bind_columns(\$r_id, \$r_refid, \$r_refpart, \$r_hasalias);
     if( $sth->fetch )
     {
-	$p->{'id'} = $r_id;
+	$p->{'uri'} = $r_id;
 
 	$obj = $self->get( $uristr );
 	$obj->[NODE][PRIVATE]{$i->[ID]} = $p;
@@ -215,17 +215,13 @@ sub add_arc
 {
     my( $self, $i, $uristr, $pred, $subj, $obj ) = @_;
 
-    if( $uristr )
-    {
-	die "Arc already exist" if $self->find_node( $uristr );
-    }
-    else
-    {
-	$uristr = $self->[NODE][URISTR].'#'.&get_unique_id;
-    }
-    my $arc_id = uri2id($uristr);
-    push @{ $subj->[NODE][PROPS]{$pred->[NODE][ID]} }, 
-      [$obj->[NODE][ID], $arc_id, $self->[NODE][ID]];
+    # Assuems that the arc does not exist
+
+    my $arc = $self->declare_arc($pred,
+				 $subj,
+				 $obj,
+				 $uristr,
+				);
 
 
     my $dbh = $i->[PRIVATE]{$i->[ID]}{'dbh'};
@@ -241,21 +237,24 @@ sub add_arc
 				      values ($place_str)
 				      ");
 
-#    my %p = %{$self->[NODE][PRIVATE]{$i->[ID]}};
+
+    # TODO: Handle if arc already exist
+    #    my %p = %{$self->[NODE][PRIVATE]{$i->[ID]}};
+
     my %p = ();
     $p{'id'}     ||= &_nextval($dbh);
 
-    $p{'uri'}         = &_create_uri( $uristr, $i);
+    $p{'uri'}         = &_create_uri( $arc->uri, $i);
     $p{'iscontainer'} = 'false';
     $p{'isprefix'}    = 'false';
     $p{'label'}       = undef;
     $p{'aliasfor'}    = undef;
-    $p{'pred'}        = &_get_id( $pred, $i);
+    $p{'pred'}        = &_get_id( $arc->pred, $i);
     $p{'distr'}       = 'false';
-    $p{'subj'}        = &_get_id( $subj, $i);
-    $p{'obj'}         = &_get_id( $obj, $i);
+    $p{'subj'}        = &_get_id( $arc->subj, $i);
+    $p{'obj'}         = &_get_id( $arc->obj, $i);
     $p{'fact'}        = 'true';
-    $p{'model'}       = &_get_id( $self, $i);
+    $p{'model'}       = &_get_id( $arc->model, $i);
     $p{'member'}      = undef;
     $p{'updated'}     = undef;
     $p{'readonly'}    = undef;
@@ -266,21 +265,22 @@ sub add_arc
     $p{'value'}       = undef;
 
 
-    $sth->execute( map $p{$_}, @node_fields ) 
+    $sth->execute( map $p{$_}, @node_fields )
 	or confess( $sth->errstr );
 
     return( 1, 1 );
 }
 
-sub init_props
+sub init_rev_subjs
 {
     my( $self, $i, $constraint ) = @_;
 
+    # This should initiate all props from this interface
+
+
     # TODO: Use the constraint
 
-    # Defined but empty props marks that the props has been initiated
-    $self->[NODE][PROPS] ||= {};
-    $self->[NODE][TYPES] or $self->init_types;
+    $self->[NODE][TYPE_ALL] or $self->init_types;
 
     # TODO: Should props be undef if type changes?
 
@@ -321,24 +321,27 @@ sub init_props
 	my $subj   = $self;
 	my $obj    = $self->get( $r->{'obj'} );
 	my $model  = $self->get( $r->{'model'} );
-	debug "..Found a $pred->[URISTR]\n", 1;
+	debug "..Found a $pred->[NODE][URISTR]\n", 1;
 
-	$subj->declare_add_prop( $pred, $obj, $model, $r->{'arc'} );
+	$subj->declare_add_prop( $pred, $obj, $r->{'arc'}, $model );
     }
 
-#warn "-back from init_props\n";
+    $self->[NODE][REV_SUBJ_ALL] = 1;
+
     return( 1, 3 );
 }
 
-sub init_rev_props
+sub init_rev_objs
 {
     my( $self, $i, $constraint ) = @_;
 
+    # This should get all rev_props from this interface
+
+
     # TODO: Use the constraint
 
-    # Defined but empty rev_props marks that the rev_props has been initiated
-    $self->[NODE][REV_PROPS] ||= {};
-    $self->[NODE][TYPES] or $self->init_types;
+
+    $self->[NODE][TYPE_ALL] or $self->init_types;
 
     my $dbh = $i->[PRIVATE]{$i->[ID]}{'dbh'};
 
@@ -379,11 +382,13 @@ sub init_rev_props
 	my $subj   = $self->get( $r->{'subj'} );
 	my $obj    = $self;
 	my $model  = $self->get( $r->{'model'} );
-	debug "..Found a $pred->[URISTR]\n", 1;
+	debug "..Found a $pred->[NODE][URISTR]\n", 1;
 
-	$subj->declare_add_prop( $pred, $obj, $model, $r->{'arc'} );
+	$subj->declare_add_prop( $pred, $obj, $r->{'arc'} );
     }
 
+    $self->[NODE][REV_OBJ_ALL] = 1;
+
     return( 1, 3 );
 }
 
@@ -404,7 +409,7 @@ sub init_types
     #
     my $dbh = $i->[PRIVATE]{$i->[ID]}{'dbh'};
     my %p = %{$self->[NODE][PRIVATE]{$i->[ID]}};
-    $p{'id'} ||= &_get_id($self, $i);
+    $p{'uri'} ||= &_get_id($self, $i);
 
     my $types = [];
 
@@ -426,7 +431,7 @@ sub init_types
 	my $true = '1';
 	my  $false = '0';
 
-	$sth_node->execute( $p{'id'} );
+	$sth_node->execute( $p{'uri'} );
 	my $tbl = $sth_node->fetchall_arrayref({});
 	$sth_node->finish; # The fetchall should finish the sth implicitly
 	foreach my $r ( @$tbl )
@@ -453,8 +458,8 @@ sub init_types
 	    # aliasfor
 
 	    # model
-	    my $model = $self->[MODEL] = 
-	      &_get_node($r->{'model'}, $self, $i);
+	    my $model= &_get_node($r->{'model'}, $self, $i);
+	    $self->[NODE][MODEL]{$model->[NODE][ID]} = $model;
 
 	    # pred distr subj obj fact
 	    if( my $r_pred = $r->{'pred'} )
@@ -467,27 +472,28 @@ sub init_types
 	    # updated readonly agent source
 	    if( my $r_updated = $r->{'updated'} )
 	    {
-		push @$types, NS_L.'#Model';
+		push @$types, NS_LS.'#Model';
 
 		my $lit_uristr = $self->[NODE][URISTR]."#updated";
-		$self->declare_add_dynamic_literal(NS_L.'updated',
+		$self->declare_add_dynamic_literal(NS_LS.'updated',
 						   \$r_updated,
-						   $model,
 						   $lit_uristr,
+						   undef,
+						   $model,
 						   );
 
 
 	    # TODO: Change this to be more RDF style!
 		if(0) #if( defined $r->{'readonly'} )
 		{
-		    my $p_readonly = $self->get(NS_L.'readonly');
+		    my $p_readonly = $self->get(NS_LS.'readonly');
 		    if( ($r->{'readonly'} eq $true) or ($r->{'readonly'} eq $false) )
 		    {
 			my $bool = $r->{'readonly'}; # Copy the value
 
+			# TODO: Change this
 			$self->declare_add_static_literal($p_readonly,
 							  \$bool,
-							  $model,
 							  );
 		    }
 		    else
@@ -506,7 +512,7 @@ sub init_types
 		debug "..Literal: $self->[NODE][URISTR]\n", 2;
 		if( $r->{'value'} )
 		{
-		    $self->[NODE][VALUE] = $r->{'value'};
+		    $self->[NODE][VALUE] = \ ($r->{'value'});
 		    push @$types, NS_RDFS.'Literal';
 		}
 		else
@@ -514,7 +520,7 @@ sub init_types
 		    die "not implemented";
 		}
 	    }
-	    $self->declare_add_types( $model, $types );
+	    $self->declare_add_types( $types );
 	}
     }
 
@@ -527,7 +533,7 @@ sub init_types
               where node=? and uri.id=type and fact=TRUE
               ");
 
-	$sth_types->execute( $p{'id'} );
+	$sth_types->execute( $p{'uri'} );
 	my $tbl = $sth_types->fetchall_arrayref({});
 	$sth_types->finish;
 	foreach my $r ( @$tbl )
@@ -536,16 +542,18 @@ sub init_types
 	    my $model = &_get_node( $r->{'model'}, $self, $i );
 
 	    # Remember the record ID
-	    $type->[NODE][PRIVATE]{$i->[ID]}{'id'} = $r->{'type'};
+	    $type->[NODE][PRIVATE]{$i->[ID]}{'uri'} = $r->{'type'};
 
 	    # TODO: Maby group the types before creating them
-	    $self->declare_add_types( $model, [$type] );
+	    $self->declare_add_types( [$type] );
 	}
     }
 
     debug "Types for $self->[NODE][URISTR]\n", 1;
     debug $self->types_as_string, 1;
 
+    $self->[NODE][TYPE_ALL] = 1;
+
     return( 1, 3 );
 }
 
@@ -555,16 +563,14 @@ sub init_rev_types
     #
     # Read the types from the DBI.
 
-    # TODO: Get the implicite types from subClassOf (Handled by
-    # RDFS_200001)
-
-    debug "Init rev_types for $self->[NODE][URISTR]\n", 2;
+    # TODO: Get the implicite types from subClassOf. ( Should be
+    # handled by declare_add_rev_types )
 
     # Look for the URI in the DB.
     #
     my $dbh = $i->[PRIVATE]{$i->[ID]}{'dbh'};
     my %p = %{$self->[NODE][PRIVATE]{$i->[ID]}};
-    $p{'id'} ||= &_get_id($self, $i);
+    $p{'uri'} ||= &_get_id($self, $i);
 
     my $rev_types = [];
 
@@ -574,7 +580,7 @@ sub init_rev_types
               where type=? and uri.id=node and fact=TRUE
               ");
 
-    $sth_rev_types->execute( $p{'id'} );
+    $sth_rev_types->execute( $p{'uri'} );
     my $tbl = $sth_rev_types->fetchall_arrayref({});
     $sth_rev_types->finish;
     foreach my $r ( @$tbl )
@@ -583,12 +589,14 @@ sub init_rev_types
 	my $model = &_get_node( $r->{'model'}, $self, $i );
 
 	# Remember the record ID
-	$rev_type->[NODE][PRIVATE]{$i->[ID]}{'id'} = $r->{'node'};
+	$rev_type->[NODE][PRIVATE]{$i->[ID]}{'uri'} = $r->{'node'};
 
 	# TODO: Group the rev_types (by model) before creating them
-	$self->declare_add_rev_types( $model, [$rev_type] );
+	$self->declare_add_rev_types( [$rev_type] );
     }
 
+    $self->[NODE][REV_TYPE_ALL] = 1;
+
     return( 1, 3 );
 }
 
@@ -605,20 +613,16 @@ sub create_literal
 	# or "failed", depending on why.
 
 	# For now: Just allow models in the local namespace
-	my $ns_l = NS_L;
-	unless( $uristr =~ /$ns_l/ )
+	unless( $uristr =~ /@{[NS_LD]}/o )
 	{
 	    die "Invalid namespace for literal";
 	}
     }
-    else
-    {
-	$uristr = NS_L."/literal/". &get_unique_id;
-    }
 
     # TODO: Make sure that $self is a model
     #
-    my $literal = $self->declare_literal( $self, $uristr, $lit_str_ref );
+    my $model = $self->[WMODEL];
+    my $literal = $self->declare_literal( $lit_str_ref, $uristr );
 
 
     # NOTE from create_model: _create_node() assumes that the resource
@@ -627,7 +631,7 @@ sub create_literal
     # on a stack of objects to be flushed to the DB once we don't have
     # to wory about the response time.
     #
-    &_create_node($literal, $i, $self);
+    &_create_node($literal, $i, $model);
 
     # Return the literal object
     #
@@ -639,30 +643,14 @@ sub create_model
     my( $self, $interface, $uri ) = @_;
     #
     # We are asked to create a new resource and a new object
-    # representing that resource.  The new resource must have an URI.
-    # There must be an interface with the authority to create the new
-    # resource within the URI, as every URI has an owner.
+    # representing that resource and a context for the resource
+    # object.  The new resource must have an URI.  The creator must
+    # own the $uri namespace, as statements will be placed in it..
 
     # If no URI is supplied, one will be generated by the method
-    # create_resource(). Which interface executing this method is
-    # dependent on the IDS.
-
-    # The Interface may want to know the type of resource to create in
-    # order to generate the URI. In case the URI is supplied, it will
+    # create_resource().  In case the URI is supplied, it will
     # be validated by the appropriate interface.
 
-    # The return value should be a result object, that containt the
-    # result as well as any error information.
-
-
-    # Using this object, the model is created with the IDS of the RDF
-    # object. This means that the object can use methods from all
-    # connected interfaces.
-
-
-    # Called from Interface or Service.  $self is the object
-    # creating the model.
-
     if( $uri )
     {
 
@@ -671,27 +659,23 @@ sub create_model
 	# or "failed", depending on why.
 
 	# For now: Just allow models in the local namespace
-	my $ns_l = NS_L;
+	my $ns_l = NS_LD;
 	unless( $uri =~ /^$ns_l/ )
 	{
 	    die "Invalid namespace for model";
 	}
     }
-    else
-    {
-	# Generate the default model URI
-	my $ns_base = &default_model_ns_base;
-	my $id = &get_unique_id;
 
-	$uri = "$ns_base/$id#";
-    }
+    # The working model of the model will be the model itself.  But
+    # the model of the model will be the working model of it's parent.
 
     # What is the model of the model?  Is it the parent model
     # ($self->[MODEL]) or itself ($model) or some default
-    # (NS_L."/model/system") or maby the interface?  Answer: Its the
+    # (NS_LD."/model/system") or maby the interface?  Answer: Its the
     # parent model.  Commonly the Service object.
     #
-    my $model = $self->declare_model( $self, $uri );
+    my $model = $self->declare_model( $uri );
+    $model->[WMODEL] = $model;
 
 
     # _create_node() assumes that the resource is not yet present in
@@ -700,7 +684,7 @@ sub create_model
     # objects to be flushed to the DB once we don't have to wory about
     # the response time.
     #
-    &_create_node($model, $interface, $self);
+    &_create_node($model, $interface, $model);
 
     # Return the model object
     #
@@ -709,13 +693,11 @@ sub create_model
 
 sub remove
 {
-    my( $self, $i, $model ) = @_;
+    my( $self, $i ) = @_;
 
     # Remove node from interface. But not from the cahce.  This is
     # called from Base delete before it removes the node from cache.
 
-    die "Model not specified" unless $model;
-
     # TODO: Check that the node (with the model) actually exist in
     # this interface
 
@@ -731,7 +713,7 @@ sub remove
                     delete from node
                     where uri = ? and model = ?");
 
-    my $r_model = &_get_id( $model, $i );
+    my $r_model = &_get_id( $self->[WMODEL], $i );
     my $r_node  = &_get_id( $self,  $i );
     my %node_p = %{$self->[NODE][PRIVATE]{$i->[ID]}};
 
@@ -740,6 +722,9 @@ sub remove
     $sth_node->execute( $r_node, $r_model)
       or confess( $sth_type->errstr );
 
+    debug "Deleted $self->[NODE][URISTR] for model ".
+      $self->[WMODEL][NODE][URISTR]."\n", 1;
+
     # Remove the private information.  This removes info for all
     # models.  Not just the deleted one.
 
@@ -753,7 +738,7 @@ sub remove
 
 sub store_types
 {
-    my( $self, $i, $model, $types ) = @_;
+    my( $self, $i, $types ) = @_;
     #
     # TODO: Could store duplicate type statements. But only from
     # diffrent models. Should not store implicit types.  The calling
@@ -769,7 +754,7 @@ sub store_types
     ");
 
     my $r_node  = &_get_id($self, $i);
-    my $r_model = &_get_id($model, $i);
+    my $r_model = &_get_id($self->[WMODEL], $i);
 
     foreach my $type ( @$types )
     {
@@ -784,7 +769,7 @@ sub store_types
 
 sub remove_types
 {
-    my( $self, $i, $model, $types ) = @_;
+    my( $self, $i, $types ) = @_;
 
     my $dbh = $i->[PRIVATE]{$i->[ID]}{'dbh'};
 
@@ -794,10 +779,12 @@ sub remove_types
     ");
 
     my $r_node  = &_get_id($self, $i);
-    my $r_model = &_get_id($model, $i);
+    my $r_model = &_get_id($self->[WMODEL], $i);
 
     foreach my $type ( @$types )
     {
+	debug "  t $type->[NODE][URISTR]\n";
+
 	my $r_type = &_get_id($type, $i);
 	$sth->execute( $r_node, $r_type, $r_model )
 	    or confess( $sth->errstr );
@@ -808,11 +795,15 @@ sub remove_types
 
 sub store_props
 {
-    my( $self, $i, $model, $props ) = @_;
+    my( $self, $i, $preds ) = @_;
     #
-    # TODO: Could store duplicate type statements. But only from
-    # diffrent models. Should not store implicit props.  The calling
-    # function should only include the explicit props.
+    # The supplied preds are a list of pred_uri.  They specify the
+    # preds to store.  The arcs are already declared.  Store the arcs
+    # matching the WMODEL.  Implicit preds should not be included in
+    # the $preds list.  Preds already stored should not be included.
+
+    debug "You are now in $self\n", 1;
+    debug "..Will store ".($#$preds+1)." preds\n", 1;
 
     my $dbh = $i->[PRIVATE]{$i->[ID]}{'dbh'};
     my %p = %{$self->[NODE][PRIVATE]{$i->[ID]}};
@@ -824,22 +815,50 @@ sub store_props
                    values (?, ?, false, false, ?, ?, ?, ?, true, false)
     ");
 
-    my $r_subj  = &_get_id($self, $i);
-    my $r_model = &_get_id($model, $i);
+    my $r_subj = $p{'uri'} ||= &_get_id($self, $i);
+    my $r_model = &_get_id($self->[WMODEL], $i);
+    my $model_id = $self->[WMODEL][NODE][ID];
 
-    foreach my $pred ( keys %$props )
+    if( $DEBUG )
     {
-	my $r_pred = &_get_id($self->get($pred), $i);
-	foreach my $obj (@{$props->{$pred}})
-	{
-	    my $r_id  = &_nextval($dbh);
-	    my $r_obj = &_get_id( $obj, $i );
-	    my $r_uri = &_create_uri($self->[NODE][URISTR].
-				     '#'.&get_unique_id, $i);
+	my $pred_cnt = keys %{$self->[NODE][REV_SUBJ]};
+	debug "..Resource has $pred_cnt predicates\n", 1;
+    }
 
-	    $sth->execute( $r_id, $r_uri, $r_model,
-			   $r_pred, $r_subj, $r_obj )
-		or confess( $sth->errstr );
+    foreach my $pred_uri ( @$preds )
+    {
+	my $r_pred = &_get_id($self->get($pred_uri), $i);
+	my $pred_id = &uri2id( $pred_uri );
+
+	if( $DEBUG )
+	{
+	    debug "..Storing $pred_uri ($pred_id)\n", 1;
+	    debug "....".@{$self->[NODE][REV_SUBJ]{$pred_id}}." entries\n", 1;
+	}
+
+	foreach my $arc_node ( @{$self->[NODE][REV_SUBJ]{$pred_id}} )
+	{
+	    # Is this arc defined for the WMODEL?
+	    unless( $arc_node->[MODEL]{$model_id} )
+	    {
+		debug "The arc $arc_node->[URISTR] is not ".
+		  "defined in $self->[WMODEL][NODE][URISTR]", 1;
+		next;
+	    }
+
+	    my %pa = %{$arc_node->[PRIVATE]{$i->[ID]}};
+
+	    $pa{'id'} ||= &_nextval($dbh);
+	    $pa{'pred'} = $r_pred;
+	    $pa{'subj'} = $r_subj;
+	    $pa{'obj'} ||= &_get_id( $arc_node->[OBJ], $i );
+	    $pa{'uri'} ||= &_get_id( $self->new($arc_node), $i );
+
+	    $sth->execute( $pa{'id'}, $pa{'uri'}, $r_model,
+			   $r_pred, $r_subj, $pa{'obj'} )
+	      or confess( $sth->errstr );
+
+	    debug "..Stored arc $arc_node->[URISTR]\n", 1;
 	}
     }
 
@@ -850,7 +869,7 @@ sub store_props
 
 sub update_node
 {
-    my( $self, $i, $model ) = @_;
+    my( $self, $i ) = @_;
     #
     # See _create_node for comments
 
@@ -884,9 +903,9 @@ sub update_node
     $p{'subj'}        ||= &_get_id( $self->[NODE][SUBJ], $i);
     $p{'obj'}         ||= &_get_id( $self->[NODE][OBJ], $i);
     $p{'fact'}        = $self->[NODE][FACT]? 'true':'false';
-    $p{'model'}       ||= &_get_id( $model, $i) or die;
+    $p{'model'}       ||= &_get_id( $self->[WMODEL], $i) or die;
     $p{'member'}      ||= &_get_id( $self->[NODE][MEMBER], $i);
-    if( $self->is_a(NS_L.'#Model') )
+    if( $self->is_a(NS_LS.'#Model') )
     {
 	$p{'updated'}     = localtime->strftime('%Y-%m-%d %H:%M:%S');
 	$p{'readonly'}    = 'f';
@@ -932,12 +951,7 @@ sub _create_node
 
     # TODO: remove $model parameter and use $self->[MODEL]
 
-    # There is presently no way to store what information that belongs
-    # to which model. Then that has been implemented, the _create_node
-    # will cycle through the models and create one record per model.
-
-    debug "Creating node $self->[NODE][URISTR]\n", 2;
-#    warn $self->to_string;
+    debug "_create_node $self->[NODE][URISTR]\n", 2;
 
     # Interface PRIVATE data. These has to be updated then the
     # corresponding official data change. The dependencies could be
@@ -946,7 +960,7 @@ sub _create_node
     my %p = %{$self->[NODE][PRIVATE]{$i->[ID]}};
 
     debug "Getting DBH for $i->[URISTR] from ".
-	"[PRIVATE]{$i->[ID]}{'dbh'}\n", 2;
+	"[PRIVATE]{$i->[ID]}{'dbh'}\n", 3;
     my $dbh = $i->[PRIVATE]{$i->[ID]}{'dbh'};
 
 
@@ -994,7 +1008,7 @@ sub _create_node
     $p{'fact'}        = $self->[NODE][FACT]? 'true':'false';
     $p{'model'}       ||= &_get_id( $model, $i) or die;
     $p{'member'}      ||= &_get_id( $self->[NODE][MEMBER], $i);
-    if( $self->is_a(NS_L.'#Model') )
+    if( $self->is_a(NS_LS.'#Model') )
     {
 	$p{'updated'}     = localtime->strftime('%Y-%m-%d %H:%M:%S');
 	$p{'readonly'}    = 'f';
@@ -1019,6 +1033,9 @@ sub _create_node
 	$p{'isliteral'}   = 'false';
     }
 
+    debug ".. id: $p{'id'}\n", 1;
+    debug "..uri: $p{'uri'}\n", 1;
+
 #    confess "SQL insert node $self->[NODE][URISTR]\n" if $DEBUG;
 
     $sth->execute( map $p{$_}, @node_fields )
@@ -1051,8 +1068,6 @@ sub _get_node
     $sth->bind_columns(\$r_uristr, \$r_refid, \$r_refpart, \$r_hasalias);
     if( $sth->fetch )
     {
-	$p->{'id'} = $r_id;
-
 	$obj = $caller->get( $r_uristr );
 	$obj->[NODE][PRIVATE]{$i->[ID]} = $p;
     }
@@ -1069,14 +1084,14 @@ sub _get_id
     my( $obj, $interface ) = @_;
     #
     # The object already exist.  Here we just want to know what id it
-    # has in the DB
+    # has in the DB. NB!!! field URI in NODE table.
 
     debug "_get_id( $obj->[NODE][URISTR]\n", 2;
 
     # Has the object a known connection to the DB?
     #
     my $p = $obj->[NODE][PRIVATE]{$interface->[ID]} || {};
-    if( defined( my $id = $p->{'id'}) )
+    if( defined( my $id = $p->{'uri'}) )
     {
 	return $id;
     }
@@ -1098,7 +1113,7 @@ sub _get_id
     $sth->bind_columns(\$r_id, \$r_refid, \$r_refpart, \$r_hasalias);
     if( $sth->fetch )
     {
-	$p->{'id'} = $r_id;
+	$p->{'uri'} = $r_id;
 	$sth->finish; # Release the handler
 
 	# TODO: Maby update other data with the result?
@@ -1114,15 +1129,7 @@ sub _get_id
 	# inserted since it would already be in the DB if this interface
 	# handles its storage.
 
-	my $sth = $dbh->prepare_cached("
-                  insert into uri
-                  (string, id, hasalias)
-                  values (?,?,false)
-                  ");
-	$r_id = &_nextval($dbh, 'uri_id_seq');
-	$sth->execute($obj->[NODE][URISTR], $r_id);
-	$sth->finish;
-	return $r_id;
+	return &_create_uri( $obj->[NODE][URISTR], $interface );
     }
 }
 
@@ -1167,10 +1174,4 @@ sub _nextval
     $id or die "Failed to get nextval";
 }
 
-sub default_model_ns_base
-{
-    return NS_L."/model";
-}
-
-
 1;
